package jp.co.powerbeans.vbscore.db;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;

import jp.co.powerbeans.common.ChkConv;
import jp.co.powerbeans.vbscore.model.Game;

import org.apache.commons.beanutils.BeanUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;


/**
 * Model,Table周りの変換Util
 * @author amonden
 * @create date 2006/09/15
 */
public class DBUtil {

	/**
	 * modelsの各columns[]値をString配列にし、それを行数分のString[][]にして返す
	 * @param models モデル配列
	 * @param columns 表示カラム
	 * @return 表示用String[][]
	 */
	public static String[][] modelsToStrings(Object[] models, String[] columns) {
		
		if (models == null || columns == null) {
			return new String[][]{{}};
		}
//		List res = new ArrayList(models.length);
		String[][] res = new String[models.length][columns.length];
		
		try {
			// プロパティマップ作成
			Map<String, PropertyDescriptor> propmap = new HashMap<String, PropertyDescriptor>();
			
            if (models.length > 0) {
                BeanInfo info = Introspector.getBeanInfo(models[0].getClass());
                PropertyDescriptor[] properties = info.getPropertyDescriptors();
                for (int i = 0; i < properties.length; i++) {
					if (properties[i].getName().equals("class")) {
						continue;
					}
					propmap.put(properties[i].getName(), properties[i]);
				}
            }
            
            // 値を取得
			for (int i = 0; i < models.length; i++) {
				for (int j = 0; j < columns.length; j++) {
					String val = BeanUtils.getProperty(models[i], columns[j]);

					// データタイプにより変換
					if (propmap.containsKey(columns[j])) {
						PropertyDescriptor pd = (PropertyDescriptor) propmap.get(columns[j]);
						Class<?> typ = pd.getPropertyType();
						
						if (typ.isAssignableFrom(Date.class)) {
							// Date型の場合 yyyy/MM/dd に変換
							val = ChkConv.toString((Date)pd.getReadMethod().invoke(models[i],null));
						}
					}
					if (val == null) {
						val = "";
					}
					res[i][j] = val;
				}
			}
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		} catch (IntrospectionException e) {
			e.printStackTrace();
		}

		return res;
	}

	/**
	 * オブジェクト配列をComboBoxModelに変換して返す
	 * @param models モデル配列
	 * @return ComboBoxModel
	 */
	public static ComboBoxModel modelsToComboBoxModel(Object[] models) {
		return modelsToDefaultComboBoxModel(models);
	}

	/**
	 * オブジェクト配列をDefaultComboBoxModelに変換して返す
	 * @param models モデル配列
	 * @return DefaultComboBoxModel
	 */
	public static DefaultComboBoxModel modelsToDefaultComboBoxModel(Object[] models) {
		return new DefaultComboBoxModel(models);
	}

	/**
	 * オブジェクト配列をDefaultComboBoxModelに変換して返す
	 * @param models モデル配列
	 * @param cmb_model デフォルトコンボボックス
	 * @return DefaultComboBoxModel
	 */
	public static DefaultComboBoxModel modelsToDefaultComboBoxModel(Object[] models, DefaultComboBoxModel cmb_model) {
		cmb_model.removeAllElements();
		for (int i = 0; i < models.length; i++) {
			cmb_model.addElement(models[i]);
		}
		return cmb_model;
	}

	/**
	 * SWT TableにString[][]を表示
	 * @param 表示先Table
	 * @param datas 表示Data
	 */
	public static void modelsToSwtTable(Table table, String[][] datas, Object[] objects) {
		for (int i = 0; i < datas.length; i++) {
	        TableItem item = new TableItem(table, SWT.NULL);
	        item.setText(datas[i]);
	        if (objects != null) {
	        	item.setData(objects[i]);
	        }
		}
	}

	/**
	 * SWT TableにString[][]を表示
	 * @param 表示先Table
	 * @param datas 表示Data
	 */
	public static void modelsToSwtTable(Table table, String[][] datas) {
		modelsToSwtTable(table, datas, (Object[])null);
	}
	
	/**
	 * SWT Tableに検索結果を格納し,Tableにobjectsを保持しておく
	 * @param table
	 * @param objects
	 * @param columns
	 */
	public static void modelsToSwtTable(Table table, Object[] objects, String[] columns) {
		String[][] datas = DBUtil.modelsToStrings(objects, columns);
		modelsToSwtTable(table, datas, objects);
		
		// 検索結果のobjectsもそのまま格納
		table.setData(objects);
	}

	/**
	 * SWT Comboにmodelsのdisp_colプロパティ値をラベルとして追加し
	 * ComboのDataにmodelsを格納する
	 * @param combo
	 * @param models
	 * @param disp_col
	 */
	public static void modelsToSwtCombobox(Combo combo, Object[] models, String disp_col) {
		String[][] datas = modelsToStrings(models, new String[]{disp_col});
		combo.removeAll();
		
		// 表示文字列を設定
		for (int i = 0; i < datas.length; i++) {
			combo.add(datas[i][0]);
		}
		// 検索結果モデルをそのまま格納
		combo.setData(models);
	}

	/**
	 * HSQLDB用Date型比較SQL文を生成して返す
	 * @param dt
	 * @param add_date
	 * @return
	 */
	public static String formatDate(Date dt, int add_date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(dt);
		cal.add(Calendar.DAY_OF_MONTH, add_date);
		
		SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.0");
		return df.format(cal.getTime());
	}
	
	/**
	 * dt を yyyy/MM/dd HH:mm:ss.0 形式で返す
	 * @param dt 日付
	 * @return フォーマット結果
	 */
	public static String formatDate(Date dt) {
		return formatDate(dt, 0);
	}

	/**
	 * colname >= date 句を生成して返す
	 * @param colname 列名
	 * @param date 日付
	 * @return 作成SQL式
	 */
	public static String sqlFromDate(String colname, Date date) {
		return " " + colname + " >= '" + ChkConv.toStringHFun(date) + "' ";
	}

	/**
	 * colname <= date 句を生成して返す
	 * @param colname 列名
	 * @param date 日付
	 * @return 作成SQL式
	 */
	public static String sqlToDate(String colname, Date date) {
		return " " + colname + " <= '" + ChkConv.toStringHFun(date) + "' ";
	}

	/**
	 * SQLの日付検索条件を作成
	 * @param colname 列名
	 * @param from_dt 開始日
	 * @param to_dt 終了日
	 * @return 作成SQL式
	 */
	public static String sqlDateBetween(String colname, Date from_dt, Date to_dt) {
		StringBuffer sub = new StringBuffer();
		if (from_dt != null) {
			sub.append(" AND " + DBUtil.sqlFromDate(colname, from_dt));
		}
		if (to_dt != null) {
			sub.append(" AND " + DBUtil.sqlToDate(colname, to_dt));
		}
		return sub.toString();
	}

	/**
	 * objs配列の各Beanのpropnameプロパティの値が重複する場合は
	 * 2つ目以降の値を""にする.
	 * 
	 * @param objs Bean配列
	 * @param propname 値チェック用プロパティ名
	 */
	public static void clearDuplicate(Object[] objs, String propname) {
		try {
			String last_name = "";
			if (objs.length > 0) {
				last_name = BeanUtils.getProperty(objs[0], propname);
			}
			for (int i = 1; i < objs.length; i++) {
				if (last_name.equals(BeanUtils.getProperty(objs[i], propname))) {
					BeanUtils.setProperty(objs[i], propname, "");
				} else {
					last_name = BeanUtils.getProperty(objs[i], propname);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * objs配列の各Beanのpropnameプロパティの値が matchstr と一致する場合は
	 * replacestr に置換する
	 * @param objs Bean配列
	 * @param propname 値チェック用プロパティ名
	 * @param matchstr マッチ文字列
	 * @param replacestr 置換文字列
	 */
	public static void replace(Object[] objs, String propname, String matchstr, String replacestr) {
		try {
			for (int i = 0; i < objs.length; i++) {
				String val = BeanUtils.getProperty(objs[i], propname);
				if (val == matchstr || ( val != null && val.equals(matchstr))) {
					BeanUtils.setProperty(objs[i], propname, replacestr);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * vals 配列を IN 句に変換する
	 * @param vals 値配列
	 * @return 作成したIN句SQL
	 */
	public static String sqlToIn(Object[] vals) {
		StringBuffer buf = new StringBuffer("(");
		for (int i = 0; i < vals.length; i++) {
			if (i > 0) {
				buf.append(",");
			}
			buf.append("'" + vals[i] + "'");
		}
		buf.append(")");
		return buf.toString();
	}

	/**
	 * vals 配列を IN 句に変換する
	 * @param vals 値配列
	 * @return 作成したIN句SQL
	 */
	public static String sqlToIn(int[] vals) {
		StringBuffer buf = new StringBuffer("(");
		for (int i = 0; i < vals.length; i++) {
			if (i > 0) {
				buf.append(",");
			}
			buf.append(vals[i]);
		}
		buf.append(")");
		return buf.toString();
	}

	/**
	 * objs 配列の各 prop_name プロパティを IN 句に変換する
	 * @param objs
	 * @param prop_name
	 * @param sql_col
	 * @return
	 */
	public static String sqlToIn(Object[] objs, String prop_name, String sql_col) {
		
		if (objs == null || objs.length == 0) {
			return "";
		}
		
		StringBuffer buf = new StringBuffer(" AND " + sql_col + " IN (");
		for (int i = 0; i < objs.length; i++) {
			if (i > 0) {
				buf.append(",");
			}
			try {
				buf.append(BeanUtils.getProperty(objs[i], prop_name));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		buf.append(")");
		return buf.toString();
	}


}
