package jp.co.powerbeans.vbscore.swt;

import java.util.Date;

import jp.co.powerbeans.common.StringArrayFactory;
import jp.co.powerbeans.common.swt.ChkConvSwt;
import jp.co.powerbeans.vbscore.CMD;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.db.DBCacheManager;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.db.DBUtil;
import jp.co.powerbeans.vbscore.logic.DocumentMgr;
import jp.co.powerbeans.vbscore.logic.ScoreMgr;
import jp.co.powerbeans.vbscore.model.Player;
import jp.co.powerbeans.vbscore.model.PlayerRanking;
import jp.co.powerbeans.vbscore.model.RankingDef;
import jp.co.powerbeans.vbscore.model.Team;
import jp.co.powerbeans.vbscore.swt.parts.GameMiniTableComposit;

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

/**
 * ランキング分析画面
 * @author amonden
 *
 */
public class AnalayzeRankingSwt extends PBSwt {

	/**
	 * 分析結果詳細テーブル
	 */
	private Table table;
	/**
	 * チーム選択コンボ
	 */
	private Combo teamCombo;
	/**
	 * 開始日
	 */
	private Text fromDate;
	/**
	 * 終了日
	 */
	private Text toDate;
	/**
	 * 対象試合表示ミニテーブル
	 */
	private GameMiniTableComposit gameTable;

	/**
	 * table 列数
	 */
	static final int COLS = 8;
	/**
	 * table 各列の列幅
	 */
	static final int COLS_WIDTH = 120;
	/**
	 * ウィンドウ幅
	 */
	static final int WINDOW_WIDTH = COLS * COLS_WIDTH;
	/**
	 * ウィンドウ高さ
	 */
	static final int WINDOW_HEIGTH = 700;

	/**
	 * タイトル
	 */
	private static final String TITLE = Msg.getPlayRanking();
	/**
	 * トップレベルメニュー ("出力")
	 */
	private static final String[] MENU_LABELS1 = {Msg.getMenuOutput()};
	/**
	 * サブレベルメニュー("HTML")
	 */
	private static final String[][] MENU_LABELS2 = {
		{Msg.getMenuHtml()
			}
	};
	/**
	 * サブレベルメニューコマンド("HTML")
	 */
	private static final String[][] MENU_CMDS = {
		{CMD.OUTPUT_HTML 
			}
	};

	/**
	 * コンストラクタ
	 */
	public AnalayzeRankingSwt() {
		super();
		initialize();
	}	
	
	/**
	 * 初期化
	 */
	private void initialize() {
		createSShell(TITLE, WINDOW_WIDTH, WINDOW_HEIGTH, true);
		setGridLayout(8);
		createItems();
		createMenu();
	}

	/**
	 * Item作成
	 */
	private void createItems() {
		addLabels(COLS, COLS_WIDTH);
		
		// チーム選択コンボ
		setHorizontalSpan(addLabel_right(Msg.getTargetTeam()), 2);
		this.teamCombo = addComboBoxByModel(Team.class, "teamName", CMD.SEL_CHANGE);
		setHorizontalSpan(this.teamCombo, 4);
		selectTop(this.teamCombo);
		addLabels(2, COLS_WIDTH);
		
		// 対象期間
		setHorizontalSpan(addLabel_right(Msg.getTargetGameDateSpan()), 2);
		this.fromDate = addTextBox("fromDate");
		addLabel_hvcenter(Msg.getChilda());
		this.toDate = addTextBox("toDate");
		
		addLabels(1, COLS_WIDTH);
		addSimpleButton(Msg.getSearchW(), CMD.SEARCH);
		addLabels(2, COLS_WIDTH);

		// 対象試合テーブル作成
		createGameTable();
		
		// 詳細テーブル作成
		createTable();
	}

	/**
	 * gameTable作成
	 */
	private void createGameTable() {
		if (gameTable == null) {
			this.gameTable = new GameMiniTableComposit(this.sShell, COLS, false);
		}
	}

	/**
	 * table生成
	 */
	private void createTable() {
		if (table == null) {
			// ランキングdefからTableColを作成
			RankingDef[] rd = (RankingDef[]) DBCacheManager.findByAll(RankingDef.class);
			String[] table_cols = new String[rd.length * 2 + 1];
			table_cols[0] = Msg.getOrder();
			for (int i = 0; i < rd.length; i++) {
				table_cols[i * 2 + 1] = rd[i].getRankingName();
				table_cols[i * 2 + 2] = Msg.getCountOrPercent();
			}
			// 列幅作成
			int[] table_col_width = new int[table_cols.length];
			table_col_width[0] = 30;
			for (int i = 1; i < table_col_width.length; i+=2) {
				table_col_width[i] = 100;
				table_col_width[i + 1] = 60;
			}
			
			table = addSimpleTable(table_cols, table_col_width, 200);
			setHeight(table, 300);
			setHorizontalSpan(table, COLS);
		}
	}

	/**
	 * 更新テーブル
	 * @param ta プレイヤーランキング
	 */
	private void updateTable(PlayerRanking[][] ta) {
		this.table.removeAll();
		addTable(ta);
	}

	/**
	 * ランキングをtableに追加
	 * @param ta プレイヤーランキング
	 */
	private void addTable(PlayerRanking[][] ta) {
		DBUtil.modelsToSwtTable(table, createTableData(ta));
	}

	/**
	 * プレイヤーランキングをテーブル表示用Dataに変換
	 * @param ta プレイヤーランキング
	 * @return テーブル表示用Dataに変換
	 */
	private String[][] createTableData(PlayerRanking[][] ta) {
		// 1行:プレー名, 2行～:1位氏名,ポイント等
		// 1,2列目:1つ目のプレー, 3,4列目:2つ目のプレー
		String[][] datas = StringArrayFactory.createEmptyString(20, ta.length * 2 + 1);
		
		int i = 0;
		// 2列目～:選手,ポイント
		{
			for (int j = 0; j < ta.length; j++) {
				for (int k = 0; k < ta[j].length; k++) {
					PlayerRanking r = ta[j][k];
					datas[i + k][j * 2 + 1] = r.getName();
					datas[i + k][j * 2 + 2] = r.getTimes() + "/" + r.getPser();
				}
			}
		}
		// 1列目:順位
		if (ta.length > 1 && ta[0].length > 0){
			int player_num = DBManager.countBy(Player.class, " teamId = (select teamID from Player where playerId = " + ta[0][0].getPlayerId() + ")");
			for (int j = 0; j < player_num; j++) {
				datas[j][0] = "" + (j + 1) + Msg.getPos();
			}
		}
		
		return datas;
	}

	/**
	 * メニュー作成
	 */
	private void createMenu() {
		create2LayerMenu(MENU_LABELS1, MENU_LABELS2, MENU_CMDS);
	}
	
	/**
	 * HTML出力メニュー選択時
	 */
	public void cmdOutput_html() {
		int team_id = Integer.parseInt(ChkConvSwt.getSelectObjectPK(this.teamCombo));
		DocumentMgr.outputAnalyzeRanking(this.sShell, team_id, this.table, this.gameTable.getTable());
	}
	
	/**
	 * ドキュメントを開く選択時
	 */
	public void cmdOutput_open_document() {
		// TODO user JOOConverter
	}
	
	/**
	 * 検索ボタン選択時
	 */
	public void cmdSearch() {
		// 検索条件取得
		int team_id = Integer.parseInt(ChkConvSwt.getSelectObjectPK(this.teamCombo));
		Date from_dt = ChkConvSwt.getDate(this.fromDate.getText());
		Date to_dt = ChkConvSwt.getDate(this.toDate.getText());

		// 検索開始
		// 1. 全プレーのランキング
		PlayerRanking[][] ta = ScoreMgr.analyzePlayerRanking(team_id, from_dt, to_dt);
		
		// 2. 対象試合更新
		this.gameTable.updateByDate(team_id, from_dt, to_dt);
		
		// 更新
		updateTable(ta);
	}
	/**
	 * チーム選択コンボ選択時
	 */
	public void cmdSel_change() {
	}
}
